#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Added comments, simplied code.	KBJ 30-Oct-2007
//
// For a FULL description of this indicator and how to use it,
// please see:  http://www.kumotrader.com/ichimoku_wiki/
//
// Revised to display the same colors for SenkouSpanA&B as found in above description.	31-Oct-2007

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Ichimoku Kinko Hyo - \"Equilibrium Chart at a Glance\" - For a FULL description of this indicator and how to use it, please see:  http://www.kumotrader.com/ichimoku_wiki/")]
    [Gui.Design.DisplayName("Ichimoku Kinko Hyo")]
    public class Ichimoku : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int periodFast = 9;			// Default setting for PeriodFast
            private int periodMedium = 26;		// Default setting for PeriodMedium
            private int periodSlow = 52;		// Default setting for PeriodSlow
        // User defined variables (add any user defined variables below)
		
			private Color bandAreaColorUp = Color.Blue;
			private Color bandAreaColorDown = Color.Red;
			private int bandAreaColorOpacity = 2;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot( new Pen(Color.Red, 2),	    PlotStyle.Line, "TenkanSen"));	// Short term momentum  (short term average of highest-high & lowest-low)
            Add(new Plot( new Pen(Color.Blue, 2),	    PlotStyle.Line, "KijunSen"));	// Medium term momentum (medium term average of highest-high & lowest-low)
            Add(new Plot( new Pen(Color.DarkViolet, 2), PlotStyle.Line, "ChikouSpan"));	// Lagging price (for visual comparison with current price)
			
			Add( new Plot( new Pen(Color.Black, 1),  "SenkouSpanA" ));		// Predicts levels of future support/resistance  (Values[3])
			Add( new Plot( new Pen(Color.Gray, 1),   "SenkouSpanB" ));		// Predicts levels of future support/resistance	 (Values[4])
			
            CalculateOnBarClose	= true;						// Updated once per bar (as opposed to on every tick.)
            Overlay				= true;						// Display on top of the price in panel # 1.
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ((CurrentBar < PeriodMedium) || (CurrentBar < PeriodFast))
				return;										// Wait until we have enough bars.

			//
			// Kijun means "trend" in Japanese. Consequently, if the trend line is heading down, then this gives a 
			// selling signal and if the kijun line is advancing, then this suggests a buying signal.
			//
			//     Trend (Kijun) Line = (highest high+ lowest low)/2 for the past 26 days.
			//
			KijunSen.Set(  ( MAX(High,PeriodMedium)[0] + MIN(Low,PeriodMedium)[0] ) / 2);	// Medium term momentum.

			// The Tenkan line is a signal line that works in conjunction with the trend line (kijun). A crossover
			// above the trend line gives a buy signal and a cross over below the selling line provides a sell signal.
			//
			//		Signal Line (Tenkan) = (highest high+ lowest low)/2 for the past 9 days.
			//
			TenkanSen.Set( ( MAX(High,PeriodFast)[0]   + MIN(Low,PeriodFast)[0]   ) / 2 );	// Short term momentum.

			// The Chiku line, or the lagging line, is very important for the entire Ichimoku outlook.  The lagging line
			// is simply the current close plotted 26 periods behind.
			//
			ChikouSpan.Set( PeriodMedium, Close[0] );		// Plot current price in the past, for easy visual comparison.

			// The two Senkou, or leading lines, create a Cloud-like formation ("Kumo" in Japanese), and this is an area
			// of support or resistance. The market must break above the Cloud to give a buy signal or below the Cloud to
			// give a sell signal. The Senkou lines are used in a similar manner as the standard support and resistance levels.
			//
			//		The leading line A = (Tenkan line + Kijun line)/2, plotted 26 periods ahead
			//		The leading line B = (Highest high + Lowest low)/2 for the past 52 periods, plotted 26 days ahead
			//
			// The following is a "cloud display" that shows what support/resistance values might be based on prior hi/low averages.
			// The cloud is defined by span-A & span-B.  Span-A is the average of the TenkanSen & Kijunsen, but shifted PeriodMedium
			// bars forward in time, and Span-B is the long-term (slow) average of the lowest-low & highest-high, shifted forward as well.
			//
			if ((CurrentBar < PeriodFast+PeriodMedium) || (CurrentBar < PeriodMedium+PeriodMedium) || (CurrentBar < PeriodSlow+PeriodMedium)) return;

			SenkouSpanA.Set( ( MAX(High,PeriodFast)[PeriodMedium] + MIN(Low,PeriodFast)[PeriodMedium]
							 + MAX(High,PeriodMedium)[PeriodMedium] + MIN(Low,PeriodMedium)[PeriodMedium] ) / 4 );

			SenkouSpanB.Set( ( MAX(High,PeriodSlow)[PeriodMedium] + MIN(Low,PeriodSlow)[PeriodMedium] ) / 2 );
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TenkanSen
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries KijunSen
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ChikouSpan
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SenkouSpanA
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SenkouSpanB
        {
            get { return Values[4]; }
        }

        [Description("The number 9 represents a week and a half")]
        [Category("Parameters")]
        public int PeriodFast
        {
            get { return periodFast; }
            set { periodFast = Math.Max(2, value); }
        }

        [Description("The number 26 represents standard Japanese work month")]
        [Category("Parameters")]
        public int PeriodMedium
        {
            get { return periodMedium; }
            set { periodMedium = Math.Max(4, value); }
        }

        [Description("The number 52 represents two months")]
        [Category("Parameters")]
        public int PeriodSlow
        {
            get { return periodSlow; }
            set { periodSlow = Math.Max(6, value); }
        }
        #endregion
		
		//
		//  This Plot() method has been tweaked so it will color in two different colors
		//  in-between the two plots of Values[3] and Values[4].
		//
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			base.Plot(graphics, bounds, min, max);

			if (Bars == null || ChartControl == null)
				return;

			SolidBrush brush;								// Set current brush color here.

			SolidBrush brushUP	 = new SolidBrush(Color.FromArgb(bandAreaColorOpacity*20,bandAreaColorUp));
			SolidBrush brushDOWN = new SolidBrush(Color.FromArgb(bandAreaColorOpacity*20,bandAreaColorDown));

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			SmoothingMode oldSmoothingMode = graphics.SmoothingMode;
			GraphicsPath path = new GraphicsPath();

			DataSeries series0 = (DataSeries) Values[3];	// Color in between these two plots.
			DataSeries series1 = (DataSeries) Values[4];

			brush = brushUP;								// Start with the upwards color.
			int barcount = 0;								// Start with leftmost bar.
			bool firstbar = true;							// Plotting the first bar.
			
			while (barcount < ChartControl.BarsPainted)		// Continue until all bars have been painted.
			{
				int count = 0;								// Counter for innner loop.
				for (int seriesCount = 3; seriesCount < 5; seriesCount++)
				{
					int	lastX = -1;
					int	lastY = -1;
					DataSeries	series = (DataSeries) Values[seriesCount];
					Gui.Chart.Plot plot = Plots[seriesCount];
	
					for (count = barcount; count < ChartControl.BarsPainted; count++)
					{
						int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
						if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
							continue;

						double val	= series.Get(idx);				// Get next y-value to be plotted.
						if (val == 0)								// If nothing to plot...
							continue;								// ...ignore the enrtry.
	
						int	x = (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
							  + (count - ChartControl.BarsPainted + 1) * ChartControl.BarSpace) + 1;

						int	y = (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);

						double val0 = series0.Get(idx);
						double val1 = series1.Get(idx);
						if (((val0 > val1) && (brush != brushUP))	// Now going in wrong direction?
						 || ((val0 < val1) && (brush != brushDOWN)))
						{											// Yes.  Done with this loop.
							if (lastX >= 0)							// Was there a last point?
							{										// Yes.  Connect it to the position half-way to this one.
								path.AddLine( lastX - plot.Pen.Width / 2, lastY, (x +lastX - plot.Pen.Width) / 2, (lastY + y)/2);
																	// Plot vertex of cross-over of the lines (1/2 way point).
							}
							break;									// Done, exit inner loop to change color.
						}

						if (firstbar == false)						// Is this the first plotted bar of the chart?
						{											// No.  Plot all bars after the first one.
							if (count == barcount)					// First bar after direction change (and color swap)?
							{										// Yes.  Add line segment for cross-over, 1/2 bar back.
								double valm1 = series.Get(idx-1);	// Get prior y-value to be plotted.
								lastX = x - ChartControl.BarSpace/2;// Back up 1/2 a bar for x-value.
								lastY = (y + (int) ((bounds.Y + bounds.Height) - ((valm1 - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height))/2;
							}

							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);	// Connect last point to this one.
						}
						firstbar = false;							// No longer the first bar.
						lastX = x;									// Save current position for next time, so we can connect the dots.
						lastY = y;
					}
					path.Reverse();									// Go back the other direction.
				}
				graphics.SmoothingMode = SmoothingMode.AntiAlias;
				graphics.FillPath(brush, path);
				path.Reset();										// Eliminate points already colored.

				barcount = count;									// Get ready to process next segment.
				brush = (brush == brushUP) ? brushDOWN : brushUP;	// Switch colors for next segment.
			}
			graphics.SmoothingMode = oldSmoothingMode;				// Restore smoothing mode before exiting.
		}
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Ichimoku[] cacheIchimoku = null;

        private static Ichimoku checkIchimoku = new Ichimoku();

        /// <summary>
        /// Ichimoku Kinko Hyo - please see http://www.kumotrader.com/ichimoku_wiki/index.php?title=Main_Page
        /// </summary>
        /// <returns></returns>
        public Ichimoku Ichimoku(int periodFast, int periodMedium, int periodSlow)
        {
            return Ichimoku(Input, periodFast, periodMedium, periodSlow);
        }

        /// <summary>
        /// Ichimoku Kinko Hyo - please see http://www.kumotrader.com/ichimoku_wiki/index.php?title=Main_Page
        /// </summary>
        /// <returns></returns>
        public Ichimoku Ichimoku(Data.IDataSeries input, int periodFast, int periodMedium, int periodSlow)
        {
            checkIchimoku.PeriodFast = periodFast;
            periodFast = checkIchimoku.PeriodFast;
            checkIchimoku.PeriodMedium = periodMedium;
            periodMedium = checkIchimoku.PeriodMedium;
            checkIchimoku.PeriodSlow = periodSlow;
            periodSlow = checkIchimoku.PeriodSlow;

            if (cacheIchimoku != null)
                for (int idx = 0; idx < cacheIchimoku.Length; idx++)
                    if (cacheIchimoku[idx].PeriodFast == periodFast && cacheIchimoku[idx].PeriodMedium == periodMedium && cacheIchimoku[idx].PeriodSlow == periodSlow && cacheIchimoku[idx].EqualsInput(input))
                        return cacheIchimoku[idx];

            Ichimoku indicator = new Ichimoku();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.PeriodFast = periodFast;
            indicator.PeriodMedium = periodMedium;
            indicator.PeriodSlow = periodSlow;

            Ichimoku[] tmp = new Ichimoku[cacheIchimoku == null ? 1 : cacheIchimoku.Length + 1];
            if (cacheIchimoku != null)
                cacheIchimoku.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheIchimoku = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Ichimoku Kinko Hyo - please see http://www.kumotrader.com/ichimoku_wiki/index.php?title=Main_Page
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Ichimoku Ichimoku(int periodFast, int periodMedium, int periodSlow)
        {
            return _indicator.Ichimoku(Input, periodFast, periodMedium, periodSlow);
        }

        /// <summary>
        /// Ichimoku Kinko Hyo - please see http://www.kumotrader.com/ichimoku_wiki/index.php?title=Main_Page
        /// </summary>
        /// <returns></returns>
        public Indicator.Ichimoku Ichimoku(Data.IDataSeries input, int periodFast, int periodMedium, int periodSlow)
        {
            return _indicator.Ichimoku(input, periodFast, periodMedium, periodSlow);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Ichimoku Kinko Hyo - please see http://www.kumotrader.com/ichimoku_wiki/index.php?title=Main_Page
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Ichimoku Ichimoku(int periodFast, int periodMedium, int periodSlow)
        {
            return _indicator.Ichimoku(Input, periodFast, periodMedium, periodSlow);
        }

        /// <summary>
        /// Ichimoku Kinko Hyo - please see http://www.kumotrader.com/ichimoku_wiki/index.php?title=Main_Page
        /// </summary>
        /// <returns></returns>
        public Indicator.Ichimoku Ichimoku(Data.IDataSeries input, int periodFast, int periodMedium, int periodSlow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Ichimoku(input, periodFast, periodMedium, periodSlow);
        }

    }
}
#endregion
